
/*
    Title:    Timer code for RTC
    
    Software: WINAVR
    needed
    Hardware: ATmega128
*/

#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

extern long int second,min,hr,day,month,year_lo,year_hi,hrd;

unsigned char MaxDate[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

extern unsigned char dayDisplay[4], timeday[10];

extern int military;

long int dayw;


void RTC_init(void) 
{
	/* initialize timer 0 as the timer */
	/*
	 * set source as 32khz crystal on tosc1,tosc2
	 * set compare count as satalite ID
	 * set to interrupt on compare - do not enable
	 */
	/* by the book: 1. disable interrupts */
/*	TIMSK |= ((0<<OCIE0) | (0<<TOIE0));*/
	TIMSK &= ~((1<<OCIE0) | (1<<TOIE0)); //disables the interrupts just in case
	/* 2. select source by setting AS0 as appropriate. */
	/* ASSR set to use external clock */
	ASSR |= 1<<AS0;
	/* 3. set TCNT0, OCR0, TCCR0 */
	TCNT0 = 0;
	/* set the Output compare register to satelite ID-- timer counts in ~ms */
	// OCR0 = 255;
	
	/* mode=normal, OC0 (pin) not used, prescaler set to 0 
	TCCR0 = ((0<<WGM00) | (0<<WGM01) | (0<<COM00) | (0<<COM01)); */
	TCCR0 = 0;
	/* 4. wait for TCN0UB, OCR0UB, and TCR0UB (to clear??) */
	while(ASSR & ((1<<TCN0UB) | (1<<OCR0UB) | (1<<TCR0UB)));
	
	/* 5. clear the Timer/Counter0 interrupt flags */
	TIFR &= ~(1<<TOV0);
	
	
	/* enable overflow interrupt enable */
	TIMSK |= (1<<TOIE0);
	 
	 
	/* Sets the prescalar to 128 to obtain 1 interrupt per second */
	TCCR0 |=	((1<<CS2) | (1<<CS0)); //sets the bits and starts timer running
	TCCR0 &=	~(0<<CS1); //clears the bit
	
	//Initialize Time Settings
	hr = 12;
	min = 59;
	second = 0;
	day = 2;
	month = 1;
	year_lo = 2;
	year_hi = 20;
	dayw = 2;
	
}

SIGNAL(SIG_OVERFLOW0)  //SIG_OVERFLOW0 was found in the iomega128.h file
                       //SIG_OVERFLOW0 is the interupt for when TIMER0 overflows ie. exceeds 256
{
	
	second++;  //increments the second on each interrupt

}


void convert_time(void)
{
   unsigned char LeapYear = 0;
    
    //the variable SECOND gets updated in the Timer0 Overflow Interrupt Routine every second 
      
    if(second > 59)                         // if one minute
    {   
        second = 0;                         // clear SECOND
        min++;                           // increment MINUTE
    
        if(min > 59)                     // if one hour        
        {
            min = 0;                     // clear MINUTE
            hr++;						// increment HOUR
			//hrd = hr;
			
			
			
            if(hr > 23)                   // if one hour
            {
                hr = 0; 	// clear HOUR
                
				strcpy(timeday, "AM");
				
				day++;					// increment DAY
                dayw++;
				
				if (dayw == 8)
				{
					dayw=1;
				}
				
				if (dayw == 1)
				{
					strcpy(dayDisplay, "SUN");
				}
				else if(dayw == 2)
				{
					strcpy(dayDisplay, "MON");
				}
				else if(dayw == 3)
				{
					strcpy(dayDisplay, "TUE");
				}
				else if(dayw == 4)
				{
					strcpy(dayDisplay, "WED");
				}
				else if(dayw == 5)
				{
					strcpy(dayDisplay, "THU");
				}
				else if(dayw == 6)
				{
					strcpy(dayDisplay, "FRI");
				}
				else if(dayw == 7)
				{
					strcpy(dayDisplay, "SAT");
				}
				
                if(month == 2)              // if it's February
                {
                    // check for leap year
                    if(!year_lo)                        // if YEAR_LO = 0, (a century has passed)
                    {
                        if(!(year_hi%4))                // check if YEAR_HI is divisible by 4 
                            LeapYear = 1;                // then it is a leap year
                    }
                    else if(!(year_lo%4) & (year_lo != 0))  // else if YEAR_LO is divisible by 4 and not zero
                        LeapYear = 1;                    // then it's a leap year
                }                        
                
                if(day > (MaxDate[month] + LeapYear))   // if a whole month has passed
                {
                    day = 1;                // clear DAY
                    month++;                // increment MONTH
                        
                    if(month > 12)          // if one year
                    {   
                        month = 1;          // clear MONTH
                        year_lo++;          // increment YEAR_LO
                        
                        if(year_lo > 99)    // if one century
                        {
                            year_lo = 0;    // clear YEAR_LO
                            year_hi++;      // increment YEAR_HI
                                
                            if(year_hi > 99)    // if 100 centuries
                                year_hi = 0;    // the AVR is still going strong :)
                        }
                    }    
                }
            }
        }
    }  
	hrd=hr;
	if (military == 0)
	{
		if (hr > 12)
		{
			hrd = hr-12;
			strcpy(timeday, "PM");
		}
		else if(hr==0)
		{	
			hrd =12;
		}
	}
	
}

